@@ -1,10 +1,9 @@
 ##++
-##     CGI Lite v3.02_02
+##     CGI Lite v3.01
 ##
 ##     see separate CHANGES file for detailed history
 ##
-##     Changes in versions 2.03 and newer copyright
-##     (c) 2014-2015, 2017-2018, 2021 Pete Houston
+##     Changes in versions 2.03 and newer copyright (c) 2014-2015 Pete Houston
 ##
 ##     Copyright (c) 1995, 1996, 1997 by Shishir Gundavaram
 ##     All Rights Reserved
@@ -516,7 +515,7 @@ lightweight CGI implementations.
 
 =head1 REPOSITORY
 
-L<https://gitlab.com/openstrike/cgi-lite/>
+L<https://github.com/openstrike/perl-CGI-Lite>
 
 =head1 MAINTAINER
 
@@ -530,23 +529,23 @@ and offering suggestions:
 
 =over 4
 
-=item Eric D. Friedman
+=item Eric D. Friedman (friedman@uci.edu)   
 
-=item Thomas Winzig
+=item Thomas Winzig (tsw@pvo.com)
 
-=item Len Charest
+=item Len Charest (len@cogent.net)
 
-=item Achim Bohnet
+=item Achim Bohnet (ach@rosat.mpe-garching.mpg.de)
 
-=item John E. Townsend
+=item John E. Townsend (John.E.Townsend@BST.BLS.com)
 
-=item Andrew McRae
+=item Andrew McRae (mcrae@internet.com)
 
-=item Dennis Grant
+=item Dennis Grant (dg50@chrysler.com)
 
-=item Scott Neufeld
+=item Scott Neufeld (scott.neufeld@mis.ussurg.com)
 
-=item Raul Almquist
+=item Raul Almquist (imrs@ShadowMAC.org)
 
 =item and many others!
 
@@ -560,8 +559,7 @@ Smylers, Andreas, Ben and Shishir.
 Copyright (c) 1995, 1996, 1997 by Shishir Gundavaram.
 All Rights Reserved.
 
-Changes in versions 2.03 onwards are copyright 2014, 2015, 2017, 2018, 2021
-by Pete Houston.
+Changes in versions 2.03 onwards are copyright 2014, 2015 by Pete Houston.
 
 Permission to use, copy, and  distribute  is  hereby granted,
 providing that the above copyright notice and this permission
@@ -594,7 +592,7 @@ BEGIN {
 	our @EXPORT = qw/browser_escape url_encode url_decode is_dangerous/;
 }
 
-our $VERSION = '3.02_02';
+our $VERSION = '3.01';
 
 ##++
 ##  Start
@@ -1151,9 +1149,7 @@ sub _parse_multipart_data
 {
 	my ($self, $total_bytes, $boundary) = @_;
 	my $files = {};
-	my $boundary_re = qr/(.*?)((?:\015?\012)?-*
-		\Q$boundary\E
-		-*[\015\012]*)(?=(.*))/xs;
+	$boundary = quotemeta ($boundary);
 
 	eval {
 
@@ -1171,10 +1167,10 @@ sub _parse_multipart_data
 		$platform    = $self->{platform};
 		$eol         = $self->{eol}->{$platform};
 		$directory   = $self->{multipart_dir};
-		$bytes_left  = $total_bytes;
 
-		while ($bytes_left) {
-			if ($byte_count < $total_bytes) {
+		while (1) {
+			if (   ($byte_count < $total_bytes)
+				&& (length ($current_buffer || '') < ($buffer_size * 2))) {
 
 				$bytes_left = $total_bytes - $byte_count;
 				$buffer_size = $bytes_left if ($bytes_left < $buffer_size);
@@ -1208,7 +1204,8 @@ sub _parse_multipart_data
 			##  value that has the first two characters ("--") missing.
 			##--
 
-			if ($current_buffer =~ $boundary_re) {
+			if ($current_buffer =~
+				/(.*?)((?:\015?\012)?-*$boundary-*[\015\012]*)(?=(.*))/os) {
 
 				($store, $this_boundary, $old_data) = ($1, $2, $3);
 
@@ -1287,6 +1284,10 @@ sub _parse_multipart_data
 			} elsif ($old_data) {
 				$store    = $old_data;
 				$old_data = $new_data;
+
+			} else {
+				$store          = $current_buffer;
+				$current_buffer = $new_data;
 			}
 
 			unless ($changed) {
@@ -1312,7 +1313,7 @@ sub _store
 	if ($file) {
 		if ($convert) {
 			if ($platform eq 'PC') {
-				$$info =~ s/\015(?!\012)|(?<!\015)\012/$eol/og;
+				$$info =~ s/\015(?=[^\012])|(?<=[^\015])\012/$eol/og;
 			} else {
 				$$info =~ s/\015\012/$eol/og;
 				$$info =~ s/\015/$eol/og if ($platform ne 'Mac');
